'use strict';
var CPF = require("cpf_cnpj").CPF,
    CNPJ = require("cpf_cnpj").CNPJ,
    moment = require('moment');

var Boleto = module.exports = function () {

};

/**
 * Remove acentos
 * @param str
 * @returns {string}
 * @private
 */
Boleto.prototype.__removeAcentos = function (str) {
    var map = {
        "â": "a",
        "Â": "A",
        "à": "a",
        "À": "A",
        "á": "a",
        "Á": "A",
        "ã": "a",
        "Ã": "A",
        "ê": "e",
        "Ê": "E",
        "è": "e",
        "È": "E",
        "é": "e",
        "É": "E",
        "î": "i",
        "Î": "I",
        "ì": "i",
        "Ì": "I",
        "í": "i",
        "Í": "I",
        "õ": "o",
        "Õ": "O",
        "ô": "o",
        "Ô": "O",
        "ò": "o",
        "Ò": "O",
        "ó": "o",
        "Ó": "O",
        "ü": "u",
        "Ü": "U",
        "û": "u",
        "Û": "U",
        "ú": "u",
        "Ú": "U",
        "ù": "u",
        "Ù": "U",
        "ç": "c",
        "Ç": "C"
    };

    return (str || '').replace(/[\W\[\] ]/g, function (a) {
        return map[a] || a;
    });
};

/**
 * Rpad de string
 * @param str
 * @param padString
 * @param length
 * @returns {*|string}
 * @private
 */
Boleto.prototype.__rpad = function (str, padString, length) {
    str = str || '';

    if (str.length > length) {
        str = str.substr(0, length);
    }

    while (str.length < length)
        str = str + padString;
    return str;
};

/**
 * Lpad de string
 * @param str
 * @param padString
 * @param length
 * @returns {*|string}
 * @private
 */
Boleto.prototype.__lpad = function (str, padString, length) {
    str = str || '';

    if (str.length > length) {
        str = str.substr(0, length);
    }

    while (str.length < length)
        str = padString + str;
    return str;
};

/**
 * Seta o codigo do pedido
 * @param {String} pedido
 */
Boleto.prototype.setPedido = function (pedido) {
    if ((pedido || '').length < 1 || (pedido || '').length > 8) {
        this.pedido = null;
        throw new Error('Número do pedido inválido, deve ter no minimo 1 posição e no máximo 8.');
    }

    if (!parseInt(pedido)) {
        this.pedido = null;
        throw new Error('Número do pedido inválido, não pode haver caracteres');
    }

    this.pedido = pedido;
};

/**
 * Retorna o código do pedido
 * @returns {String|*}
 */
Boleto.prototype.getPedido = function () {
    return this.__lpad(this.__removeAcentos(this.pedido || ''), '0', 8);
};

/**
 * Seta o valor do boleto
 * @param {float} valor
 */
Boleto.prototype.setValor = function (valor) {
    if (typeof valor !== 'number') {
        this.valor = null;
        throw new Error('Valor precisa ser númerico');
    }

    this.valor = valor;
};

/**
 * Retorna o valor do boleto
 * @returns {float|*}
 */
Boleto.prototype.getValor = function () {
    var valor = parseFloat(this.valor || 0).toFixed(2).toString().replace(/[^0-9]/g, "");
    return this.__lpad(valor, '0', 10);
};

/**
 * Seta a observação do boleto
 * @param {String} observacao
 */
Boleto.prototype.setObservacao = function (observacao) {
    this.observacao = observacao;
};

/**
 * Retorna a observação do boleto
 * @returns {String|*}
 */
Boleto.prototype.getObservacao = function () {
    return this.__rpad(this.__removeAcentos(this.observacao || ''), ' ', 40);
};

/**
 * Seta o nome do sacado do boleto
 * @param {String} nomeSacado
 */
Boleto.prototype.setNomeSacado = function (nomeSacado) {
    this.nomeSacado = nomeSacado;
};

/**
 * Retorna o nome do sacado do boleto
 * @returns {String|*}
 */
Boleto.prototype.getNomeSacado = function () {
    return this.__rpad(this.__removeAcentos(this.nomeSacado || ''), ' ', 30);
};

/**
 * Seta o CPF/CNPJ do sacado
 * @param {String} cpfCnpj
 */
Boleto.prototype.setCpfCnpj = function (cpfCnpj) {
    this.codigoInscricao = null;
    this.cpfCnpj = null;

    if (CNPJ.isValid(cpfCnpj)) {
        cpfCnpj = CNPJ.strip(cpfCnpj);
        this.codigoInscricao = "02";
    } else if (CPF.isValid(cpfCnpj)) {
        cpfCnpj = CPF.strip(cpfCnpj);
        this.codigoInscricao = "01";
    } else {
        throw new Error("CPF/CNPJ inválido");
    }

    this.cpfCnpj = cpfCnpj;
};

/**
 * Retorna o CPF/CNPJ do sacado
 * @returns {String|*}
 */
Boleto.prototype.getCpfCnpj = function () {
    return this.__rpad((this.cpfCnpj || '').toString(), ' ', 14);
};

/**
 * Seta o endereço do sacado do boleto
 * @param {String} enderecoSacado
 */
Boleto.prototype.setEnderecoSacado = function (enderecoSacado) {
    this.enderecoSacado = enderecoSacado;
};

/**
 * Retorna o endereço do sacado do boleto
 * @returns {String|*}
 */
Boleto.prototype.getEnderecoSacado = function () {
    return this.__rpad(this.__removeAcentos(this.enderecoSacado || ''), ' ', 40);
};

/**
 * Seta o bairro do sacado do boleto
 * @param {String} bairroSacado
 */
Boleto.prototype.setBairroSacado = function (bairroSacado) {
    if ((bairroSacado || '').length > 15) {
        this.bairroSacado = null;
        throw new Error('Bairro não pode ter mais do que 15 posições');
    }
    this.bairroSacado = bairroSacado;
};

/**
 * Retorna o bairro do sacado do boleto
 * @returns {String|*}
 */
Boleto.prototype.getBairroSacado = function () {
    return this.__rpad(this.__removeAcentos(this.bairroSacado || ''), ' ', 15)
};

/**
 * Seta o CEP do sacado do boleto
 * @param {String} cepSacado
 */
Boleto.prototype.setCepSacado = function (cepSacado) {
    if ((/^[0-9]{2}[0-9]{3}-[0-9]{3}$/.test(cepSacado || '')) || (/^[0-9]{2}\.[0-9]{3}-[0-9]{3}$/.test(cepSacado || ''))) {
        this.cepSacado = cepSacado;
    } else {
        this.cepSacado = null;
        throw new Error('CEP Inválido');
    }
};

/**
 * Retorn o CEP do sacado do boleto
 * @returns {String|*}
 */
Boleto.prototype.getCepSacado = function () {
    return this.__rpad((this.cepSacado || '').replace(/[^0-9]/g, "").toString(), ' ', 8);
};

/**
 * Seta a cidade do sacado do boleto
 * @param {String} cidadeSacado
 */
Boleto.prototype.setCidadeSacado = function (cidadeSacado) {
    if ((cidadeSacado || '').length > 15) {
        this.cidadeSacado = null;
        throw new Error('Cidade não pode ter mais do que 15 posições');
    }

    this.cidadeSacado = cidadeSacado;
};

/**
 * Retorna a cidade do sacado do boleto
 * @returns {String|*}
 */
Boleto.prototype.getCidadeSacado = function () {
    return this.__rpad(this.__removeAcentos(this.cidadeSacado || ''), ' ', 15);
};

/**
 * Seta o estado do sacado do boleto
 * @param {String} estadoSacado
 */
Boleto.prototype.setEstadoSacado = function (estadoSacado) {
    if ([
            'AC',
            'AL',
            'AM',
            'AP',
            'BA',
            'CE',
            'DF',
            'ES',
            'GO',
            'MA',
            'MG',
            'MS',
            'MT',
            'PA',
            'PB',
            'PE',
            'PI',
            'PR',
            'RJ',
            'RN',
            'RO',
            'RR',
            'RS',
            'SC',
            'SE',
            'SP',
            'TO'].indexOf(estadoSacado) == -1) {
        this.estadoSacado = null;
        throw new Error('Estado inválido');
    }
    this.estadoSacado = estadoSacado;
};

/**
 * Retorna o estado do sacado do boleto
 * @returns {String|*}
 */
Boleto.prototype.getEstadoSacado = function () {
    return this.estadoSacado || '  ';
};

/**
 * Seta a data de vencimento do boleto
 * @param {Date} dataVencimento
 */
Boleto.prototype.setDataVencimento = function (dataVencimento) {
    this.dataVencimento = null;
    if (!moment(dataVencimento).isValid()) {
        throw new Error('Data de vencimento inválida');
    } else if (moment().format('YYYY-MM-DD') > moment(dataVencimento).format('YYYY-MM-DD')) {
        throw new Error('Data de vencimento deve ser maior ou igual a hoje');
    }

    this.dataVencimento = dataVencimento;
};

/**
 * Retorna a data de vencimento do boleto
 * @returns {Date|*}
 */
Boleto.prototype.getDataVencimento = function () {
    return moment(this.dataVencimento).format("DDMMYYYY");
};

/**
 * Seta a URL de retorno
 * @param {String} urlRetorno
 */
Boleto.prototype.setUrlRetorno = function(urlRetorno) {
    this.urlRetorno = urlRetorno;
};

/**
 * Retorna a URL de retorno
 * @returns {*|string}
 */
Boleto.prototype.getUrlRetorno = function() {
    return this.__rpad(this.__removeAcentos(this.urlRetorno || ''), ' ', 60);
};

/**
 * Seta a observação adicional 01 do boleto
 * @param {String} obsAd1
 */
Boleto.prototype.setObservacaoAdicional1 = function (obsAd1) {
    if ((obsAd1 || '').length > 60) {
        this.obsAd1 = null;
        throw new Error('Observação adicional 01 não pode ter mais do que 60 posições');
    }

    this.obsAd1 = obsAd1;
};

/**
 * Retorna a observação adicional 01 do boleto
 * @returns {String|*}
 */
Boleto.prototype.getObservacaoAdicional1 = function () {
    return this.__rpad(this.__removeAcentos(this.obsAd1 || ''), ' ', 60);
};

/**
 * Seta a observação adicional 02 do boleto
 * @param {String} obsAd2
 */
Boleto.prototype.setObservacaoAdicional2 = function (obsAd2) {
    if ((obsAd2 || '').length > 60) {
        this.obsAd2 = null;
        throw new Error('Observação adicional 02 não pode ter mais do que 60 posições');
    }

    this.obsAd2 = obsAd2;
};

/**
 * Retorna a observação adicional 02 do boleto
 * @returns {String|*}
 */
Boleto.prototype.getObservacaoAdicional2 = function () {
    return this.__rpad(this.__removeAcentos(this.obsAd2 || ''), ' ', 60);
};

/**
 * Seta a observação adicional 03 do boleto
 * @param {String} obsAd3
 */
Boleto.prototype.setObservacaoAdicional3 = function (obsAd3) {
    if ((obsAd3 || '').length > 60) {
        this.obsAd3 = null;
        throw new Error('Observação adicional 03 não pode ter mais do que 60 posições');
    }

    this.obsAd3 = obsAd3;
};

/**
 * Retorna a observação adicional 03 do boleto
 * @returns {String|*}
 */
Boleto.prototype.getObservacaoAdicional3 = function () {
    return this.__rpad(this.__removeAcentos(this.obsAd3 || ''), ' ', 60);
};

/**
 * Retorna se o boleto é valido
 * @returns {boolean}
 */
Boleto.prototype.isValid = function () {
    return this.pedido && this.valor && this.dataVencimento;
};

/**
 * Retorna a linha de algoritmo do boleto
 * @returns {String|*}
 */
Boleto.prototype.getLinhaBoletoAlgoritmo = function () {
    var str = this.getPedido();
    str += this.getValor();
    str += this.getObservacao();
    str += this.getNomeSacado();
    str += this.codigoInscricao;
    str += this.getCpfCnpj();
    str += this.getEnderecoSacado();
    str += this.getBairroSacado();
    str += this.getCepSacado();
    str += this.getCidadeSacado();
    str += this.getEstadoSacado();
    str += this.getDataVencimento();
    str += this.getUrlRetorno();
    str += this.getObservacaoAdicional1();
    str += this.getObservacaoAdicional2();
    str += this.getObservacaoAdicional3();

    return str;
};
